package com.ruoyi.web.yh.price.service.impl;

import java.time.LocalDateTime;
import java.util.List;

import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.web.yh.price.domain.dto.PriceVerifyDTO;
import com.ruoyi.web.yh.price.domain.enums.PriceStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.web.yh.price.mapper.PriceMapper;
import com.ruoyi.web.yh.price.domain.Price;
import com.ruoyi.web.yh.price.domain.PriceVO;
import com.ruoyi.web.yh.price.domain.PriceQuery;
import com.ruoyi.web.yh.price.service.PriceService;
import org.springframework.transaction.support.TransactionTemplate;

/**
 * 单价Service业务层处理
 *
 * @author ruoyi
 * @date 2024-10-17
 */
@Service
public class PriceServiceImpl implements PriceService
{
    @Autowired
    private PriceMapper priceMapper;
    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询单价
     *
     * @param priceId 单价主键
     * @return 单价
     */
    @Override
    public PriceVO selectPriceByPriceId(Long priceId)
    {
        return priceMapper.selectPriceByPriceId(priceId);
    }

    /**
     * 查询单价列表
     *
     * @param price 单价
     * @return 单价
     */
    @Override
    public List<PriceVO> selectPriceList(PriceQuery price)
    {
        return priceMapper.selectPriceList(price);
    }

    /**
     * 新增单价
     *
     * @param price 单价
     * @return 结果
     */
    @Override
    public int insertPrice(Price price)
    {
        price.setCreateTime(DateUtils.getNowDate());
        return priceMapper.insertPrice(price);
    }

    /**
     * 修改单价
     *
     * @param data 单价
     * @return 结果
     */
    @Override
    public int updatePrice(Price data)
    {
        PriceVO old = selectPriceByPriceId(data.getPriceId());
        ServiceUtil.assertion(old == null, "待修改的单价不存在");
        ServiceUtil.assertion(!PriceStatus.canEdit().contains(old.getStatus()), "待修改的单价当前状态不允许修改，请刷新后重试");

        data.setUpdateTime(DateUtils.getNowDate());
        Integer result = transactionTemplate.execute(status -> {
            PriceQuery query = new PriceQuery();
            query.setPriceId(data.getPriceId());
            query.setStatusList(PriceStatus.canEdit());
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "修改单价失败，当前单价已发生变化，请刷新后重试");

            return update;
        });

        return result == null ? 0 : result;
    }

    /**
     * 批量删除单价
     *
     * @param priceIds 需要删除的单价主键
     * @return 结果
     */
    @Override
    public int deletePriceByPriceIds(Long[] priceIds)
    {
        return priceMapper.deletePriceByPriceIds(priceIds);
    }

    /**
     * 删除单价信息
     *
     * @param priceId 单价主键
     * @return 结果
     */
    @Override
    public int deletePriceByPriceId(Long priceId)
    {
        return priceMapper.deletePriceByPriceId(priceId);
    }

    @Override
    public int updateByQuery(Price data, PriceQuery query) {
        if (data == null || query == null) {
            return 0;
        }
        return priceMapper.updateByQuery(data, query);
    }

    @Override
    public int submit(Long priceId) {
        PriceVO old = selectPriceByPriceId(priceId);
        ServiceUtil.assertion(old == null, "待提交的单价不存在");
        ServiceUtil.assertion(!PriceStatus.canSubmit().contains(old.getStatus()), "待提交的单价当前状态不允许提交，请刷新后重试");

        Integer result = transactionTemplate.execute(status -> {
            Price data = new Price();
            data.setStatus(PriceStatus.WAIT_VERIFY.getStatus());
            PriceQuery query = new PriceQuery();
            query.setPriceId(priceId);
            query.setStatusList(PriceStatus.canSubmit());
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "提交失败，当前单价状态已发生变化，请刷新后重试");

            return update;
        });

        return result == null ? 0 :result;
    }

    @Override
    public int cancel(Long priceId) {

        PriceVO old = selectPriceByPriceId(priceId);
        ServiceUtil.assertion(old == null, "待取消提交的单价不存在");
        ServiceUtil.assertion(!PriceStatus.canCancel().contains(old.getStatus()), "待提交的单价当前状态不允许取消提交，请刷新后重试");

        Integer result = transactionTemplate.execute(status -> {
            Price data = new Price();
            data.setStatus(PriceStatus.WAIT_SUBMIT.getStatus());
            PriceQuery query = new PriceQuery();
            query.setPriceId(priceId);
            query.setStatusList(PriceStatus.canCancel());
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "取消失败，当前单价状态已发生变化，请刷新后重试");

            return update;
        });

        return result == null ? 0 :result;
    }

    @Override
    public int verify(PriceVerifyDTO dto) {
        if (dto == null || dto.getPriceId() == null || dto.getPass() == null) {
            return 0;
        }
        PriceVO old = selectPriceByPriceId(dto.getPriceId());
        ServiceUtil.assertion(old == null, "待审核的单价不存在，请刷新后重试");
        ServiceUtil.assertion(!PriceStatus.canVerify().contains(old.getStatus()), "待审核的单价当前状态无法审核，请刷新后重试");

        Integer result = transactionTemplate.execute(status -> {
            // 修改状态
            Price data = new Price();
            data.setStatus(dto.getPass() ? PriceStatus.PASS.getStatus() : PriceStatus.REJECT.getStatus());
            data.setVerifyTime(LocalDateTime.now());
            SysUser verifyUser = dto.getVerifyUser();
            if (verifyUser != null) {
                data.setVerifyBy(verifyUser.getNickName());
                data.setVerifyId(verifyUser.getUserId());
            }
            PriceQuery query = new PriceQuery();
            query.setPriceId(dto.getPriceId());
            query.setStatusList(PriceStatus.canVerify());
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "审核失败，当前单价状态已发生变化，请刷新后重试");

            return update;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int disable(Long priceId) {
        if (priceId == null) {
            return 0;
        }
        PriceVO old = selectPriceByPriceId(priceId);
        ServiceUtil.assertion(old == null, "待禁用的单价不存在，请刷新后重试");
        ServiceUtil.assertion(!PriceStatus.canDisable().contains(old.getStatus()), "待禁用的单价当前状态不允许禁用");
        ServiceUtil.assertion(old.getDisabled() != null && old.getDisabled(), "待禁用的单价已被禁用，请勿重复操作");

        Integer result = transactionTemplate.execute(status -> {
            Price data = new Price();
            data.setDisabled(true);
            data.setDisabledTime(LocalDateTime.now());
            PriceQuery query = new PriceQuery();
            query.setStatusList(PriceStatus.canDisable());
            query.setPriceId(priceId);
            query.setDisabled(false);
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "禁用单价失败，当前状态已发生变化，请刷新后重试");

            return update;
        });

        return result == null ? 0 : result;
    }
}
